{****************************************}
{*                                      *}
{*          NXSTRUTL.PAS 2.03           *}
{*         Borland Delphi unit          *}
{* ( Version 1.X, 2.X, 3.X compatible ) *}
{*                                      *}
{*      Copyright (c) Nikita Popov      *}
{*               1997-1998              *}
{*                                      *}
{*         Code revision : 28           *}
{*         Data revision : 16           *}
{*                                      *}
{****************************************}

{- Last update : 11-08-1998 }

{$C PRELOAD MOVEABLE PERMANENT}

{$I NIXLIBS.INC}

{$A+}
{$V-}
{$X+}

unit NxStrUtl; {- string handling routines and miscs. }

interface

uses
  SysUtils;

type
  TChrSet = set of char;

type
  TCompareType = ( ctLess, ctEqual, ctGreater );

{>>A 2.03}
const
{$IFDEF WIN32}
  cbMaxFileNameLen = 64;
  cbFileNameLen    = cbMaxFileNameLen;
  cbFileExtLen     = cbMaxFileNameLen;
{$ELSE}
  cbMaxFileNameLen = 12;
  cbFileNameLen    = 8;
  cbFileExtLen     = 3;
{$ENDIF WIN32}
{<<A 2.03}

type
  TChar = array[0..256] of char;
  TExtString = string[cbFileExtLen];

const
  TFstring : array[boolean] of string[5] = ('False', 'True');

const
  NullString  : string[1] = '';
  cSpaceStr   = ' ';
  cPointStr   = '.';

const
  MoneySign : Char = '$';           {- Used by Form for floating dollar sign}

{$IFNDEF WIN32}
type
  ShortString = string;
{$ENDIF WIN32}

{/// Numeric conversion ////////////////////////////////////////////////////}

function HexB(B : Byte) : string;
{-Return hex string for byte}

function HexW(W : Word) : string;
{-Return hex string for word}

function HexL(L : LongInt) : string;
{-Return hex string for longint}

function HexPtr(P : Pointer) : string;
{-Return hex string for pointer}

function BinaryB(B : Byte) : string;
{-Return binary string for byte}

function BinaryW(W : Word) : string;
{-Return binary string for word}

function BinaryL(L : LongInt) : string;
{-Return binary string for longint}

function OctalB(B : Byte) : string;
{-Return octal string for byte}

function OctalW(W : Word) : string;
{-Return octal string for word}

function OctalL(L : LongInt) : string;
{-Return octal string for longint}

function Str2Int(const S : string; var I : Integer) : Boolean;
{-Convert a string to an integer, returning true if successful}

function Str2Word(const S : string; var I : Word) : Boolean;
{-Convert a string to a word, returning true if successful}

function Str2Long(const S : string; var I : LongInt) : Boolean;
{-Convert a string to an longint, returning true if successful}

function Str2Real(const S : string; var R : Double) : Boolean;
{-Convert a string to a real, returning true if successful.
  NOTE: for backward compatibility only. Use StrToFloat Delphi's native
  function }

function Long2Str(L : LongInt) : string;
{-Convert a longint/word/integer/byte/shortint to a string}

function Real2Str(R : Double; Width, Places : Byte) : string;
{-Convert a real to a string.
  NOTE: for backward compatibility only. Use FloatToStr Delphi's native
  function }

{/// General strings handling ///////////////////////////////////////////////}

function CharStr(Ch : Char; Len : Byte) : string;
{- Return a string of length len filled with ch}

function PadCh(const S : string; Ch : Char; Len : Byte) : string;
{- Return a string right-padded to length len with ch}

function Pad(const S : string; Len : Byte) : string;
{- Return a string right-padded to length len with blanks}

function LeftPadCh(const S : string; Ch : Char; Len : Byte) : string;
{- Return a string left-padded to length len with ch}

function LeftPad(const S : string; Len : Byte) : string;
{- Return a string left-padded to length len with blanks}

function TrimLead(const S : string) : string;
{- Return a string with removed leading blanks}

function TrimTrail(const S : string ) : string;
{- Return a string with removed triling blanks}

{$IFNDEF WIN32}
function Trim(const S : string) : string;
{- Return a string with removed leading/trailing blanks}
{$ENDIF WIN32}

function WordCount(const S : string; WordDelims : TChrSet) : longint;
{-Given a set of word delimiters, return number of words in S}

function ExtractWord(N : longint; const S : string; WordDelims : TChrSet) : string;
{-Given a set of word delimiters, return the N'th word in S}

function HasExtension(const Name : string; var DotPos : integer) : Boolean;
  {-Return whether and position of extension separator dot in a pathname}

function DefaultExtension(const Name, Ext : string) : string;
  {-Return a pathname with the specified extension attached}

function ForceExtension(const Name, Ext : string) : string;
  {-Return a pathname with the specified extension attached}

function JustExtension(const Name : string) : string;
  {-Return just the extension of a pathname}

function JustFilename(const PathName : string) : string;
  {-Return just the filename of a pathname}

function JustPathname(const PathName : string) : string;
  {-Return just the drive:directory portion of a pathname}

function AddBackSlash(const DirName : string) : string;
  {-Add a default backslash to a directory name}

function CleanFileName(const FileName : string) : string;
  {-Return filename with at most 8 chars of name and 3 of extension}

function CleanPathName(PathName : string) : string;
  {-Return a pathname cleaned up as DOS will do it}

function FullPathName(const FName : string) : string;
  {-Given FName (known to exist), return a full pathname}

{>>A 2.01}
function RemoveCharacter( const S: string; C: char ): string;
  {- Return a string, contains no C chars }
{<<A 2.01}

function CompactSpaces( S : string ):string;
  {-Return a string with all white spaces removed }

function RemoveSpaces( S : string ): string;
  {- Return a string, contains no space chars }

function digiPadCh( D: longint; Ch : Char; Len : Byte) : string;
  {-Return a string right-padded to length len with ch}

function digiPad( D: longint; Len : Byte) : string;
  {-Return a string right-padded to length len with blanks}

function digiLeftPadCh( D: longint; Ch : Char; Len : Byte) : string;
  {-Return a string left-padded to length len with ch}

function digiLeftPad( D: longint; Len : Byte) : string;
  {-Return a string left-padded to length len with blanks}

function ValidFileName( const FName : string ):boolean;

function JustDriveName( FName : string ):char;
{- return just a drive letter extracted from FName, if not present returns #0 }

function JustMainFileName( FName : string ):string;
{- return just a part of filename before point }

function CompressPath( P : string; Len : byte ): string;
{- compress the patname to form Drive:\...\[dirname]\filename if
   it's length bigger than Len }

{>> 1.11 }
function RemoveBackSlash( P : string ): string;
{- removes all back slashes from the end of P (reverse for AddBackSlash) }

function TruncLine( S : string; Len : byte ): string;
{- truncates string S by length L. If lengths of S less than Len no changes
   will be made }

function TruncLineRt( S : string; Len : byte ): string;
{- like TruncLine but trunction begins from left }

{>>A 1.1}
function CompareStrings( S1, S2 : string ): TCompareType;
{- compares two strings (case sensitive) [backward compatibility function]}

function StringsIsEqual( S1, S2 : string ): boolean;
{- returns true if strings are equal (case sensitive)
   [backward compatibility function]}

{>>A 1.12}
function FormatFIO( SrName, FName, SName : string ): string;
{- returns string formed as full surname an first letters of first name and
   secondary name delimited by points. If one of parameters is empty no point
   will be added for this parameter }

{>>A 1.14}
function CompStruct(fpBlock1, fpBlock2: Pointer; Size: Cardinal): Boolean;
{- compares two in-memory structures pointed by fpBlock1 and fpBlock2 using
   Size parameter. Returns True if these structures are equal }

{>>A 1.15}
{$IFNDEF WIN32}
procedure SetLength(var S: string; Len: byte);
{- dynamically set the length of S to Len. [for WIN16/WIN32 VCL compatibility] } 
{$ENDIF}

{>>A 2.01}
function  UpCaseFirstLtr(const S: string): string;
{- truns first letter in S to uppercase }
function  nxStrToInt(const S: string): longint;
{- like IntToStr from SysUtils but if S is '' or filled by spaces will return
   a zero, no EConvertError exception will be raised }
function  StrIsNull(const S: string): boolean;
{- returns True if length of trimmed S is zero ( length(trim(S)) = 0 )}
{<<A 2.01}

{>>A 2.02}
function  CharCount(C: char; const S: string): integer;
{- returns number of character C inside string S (case sensitive) }
procedure ReplaceChar(SrcChar, DstChar: char; var S: string);
{- replaces all occurencies of SrcChar in string S by DstChar }
function  GetQuotedString(const S: string): string;
{- returns string inside double quotes (long path names, etc.). If no one
  double qoutes apears in S then entire S will be returned as a result.
  If only one double quote appears in S all remaining characters will be
  returned as a result. }

procedure ClearTChar(var TC: TChar);

function  HexToInt(const H: string; Digits: integer; var I: longint): boolean;
{- Converts a hexadecimal string to long integer. Digits determines how
  much digits in a string must be used during conversion. Returns False if
  empty string passed or non-hex characters encountered. If error occurs,
  I variable does not changes it's value. In case of success returns True
  and decimal value in I variable. }
{<<A 2.02}

implementation

uses
  NixLibs;

type
  Long =
    record
      LowWord, HighWord : Word;
    end;

const
  HEXdigits : array[0..$F] of Char = '0123456789ABCDEF';
const
  DosDelimSet : set of Char = ['\', ':', #0];

const
  DoubleQuote = '"'; 

  function HexB(B : Byte) : string;
    {-Return hex string for byte}
  begin
    SetLength(Result, 2);
    Result[1] := HEXdigits[B shr 4];
    Result[2] := HEXdigits[B and $F];
  end;

  function HexW(W : Word) : string;
    {-Return hex string for word}
  begin
    SetLength(Result, 4);
    Result[1] := HEXdigits[hi(W) shr 4];
    Result[2] := HEXdigits[hi(W) and $F];
    Result[3] := HEXdigits[lo(W) shr 4];
    Result[4] := HEXdigits[lo(W) and $F];
  end;

  function HexL(L : LongInt) : string;
    {-Return hex string for LongInt}
  begin
    with Long(L) do
      HexL := HexW(HighWord)+HexW(LowWord);
  end;

  function HexPtr(P : Pointer) : string;
    {-Return hex string for pointer}
  begin
{$IFDEF WIN32}
    Result:= HexW(long(P).HighWord)+':'+HexW(long(P).LowWord);
{$ELSE}
    Result := HexW(Seg(P^))+':'+HexW(Ofs(P^));
{$ENDIF}
  end;

  function BinaryB(B : Byte) : string;
    {-Return binary string for byte}
  var
    I, N : Word;
  begin
    N := 1;
    SetLength(Result, 8);

    for I := 7 downto 0 do begin
      Result[N] := HEXdigits[Ord(B and (1 shl I) <> 0)]; {0 or 1}
      Inc(N);
    end;
  end;

  function BinaryW(W : Word) : string;
    {-Return binary string for word}
  var
    I, N : Word;
  begin
    N := 1;
    SetLength(Result, 16);

    for I := 15 downto 0 do begin
      Result[N] := HEXdigits[Ord(W and (1 shl I) <> 0)]; {0 or 1}
      Inc(N);
    end;
  end;

  function BinaryL(L : LongInt) : string;
    {-Return binary string for LongInt}
  var
    I : LongInt;
    N : Byte;
  begin
    N := 1;
    SetLength(Result, 32);

    for I := 31 downto 0 do begin
      Result[N] := HEXdigits[Ord(L and LongInt(1 shl I) <> 0)]; {0 or 1}
      Inc(N);
    end;
  end;

  function OctalB(B : Byte) : string;
    {-Return octal string for byte}
  var
    I : Word;
  begin
    SetLength(Result, 3);

    for I := 0 to 2 do begin
      Result[3-I] := HEXdigits[B and 7];
      B := B shr 3;
    end;
  end;

  function OctalW(W : Word) : string;
    {-Return octal string for word}
  var
    I : Word;
  begin
    SetLength(Result, 6);

    for I := 0 to 5 do begin
      Result[6-I] := HEXdigits[W and 7];
      W := W shr 3;
    end;
  end;

  function OctalL(L : LongInt) : string;
    {-Return octal string for word}
  var
    I : Word;
  begin
    SetLength(Result, 12);

    for I := 0 to 11 do begin
      Result[12-I] := HEXdigits[L and 7];
      L := L shr 3;
    end;
  end;

  function Str2Int(const S : string; var I : Integer) : Boolean;
    {-Convert a string to an integer, returning true if successful}
  begin
    Result:= True;

    try
      I:= StrToInt(S);
    except
      Result:= False;
    end; { try }
  end;

  function Str2Word(const S : string; var I : Word) : Boolean;
    {-Convert a string to a word, returning true if successful}
  begin
    Result:= True;

    try
      I:= StrToInt(S);
    except
      Result:= False;
    end; { try }
  end;

  function Str2Long(const S : string; var I : LongInt) : Boolean;
    {-Convert a string to a longint, returning true if successful}
  begin
    Result:= True;
    
    try
      I:= StrToInt(S);
    except
      Result:= False;
    end; { try }
  end;

  function Str2Real(const S : string; var R : Double) : Boolean;
    {-Convert a string to a double, returning true if successful.
      NOTE: for backward compatibility only. Use StrToFloat Delphi's native
      function }
  begin
    Result:= True;

    try
      R:= StrToFloat(S);
    except
      Result:= False;
    end; { try }
  end;

  function Long2Str(L : LongInt) : string;
    {-Convert a long/word/integer/byte/shortint to a string}
  begin
    Result:= IntToStr(L);
  end;

  function Real2Str(R : Double; Width, Places : Byte) : string;
    {-Convert a real to a string}
  begin
    Str(R:Width:Places, Result);
  end;

  function CharStr(Ch : Char; Len : Byte) : string;
    {-Return a string of length len filled with ch}
  begin
    if Len = 0 then
        SetLength(Result, 0)
      else begin
        SetLength(Result, Len);
        FillChar(Result[1], Len, Ch);
      end;
  end;

  function PadCh(const S : string; Ch : Char; Len : Byte) : string;
    {-Return a string right-padded to length len with ch}
  var
    SLen : Byte absolute S;
  begin
    if Length(S) >= Len then
      Result := S
    else begin
      SetLength(Result, Len);
      Move(S[1], Result[1], SLen);

      if SLen < 255 then
        FillChar(Result[Succ(SLen)], Len-SLen, Ch);
    end;
  end;

  function Pad(const S : string; Len : Byte) : string;
    {-Return a string right-padded to length len with blanks}
  begin
    Pad := PadCh(S, ' ', Len);
  end;

  function LeftPadCh(const S : string; Ch : Char; Len : Byte) : string;
    {-Return a string left-padded to length len with ch}
  var
{$IFNDEF WIN32}
    o : string;
    SLen : Byte absolute S;
{$ELSE}
    SLen: integer;
{$ENDIF}
  begin
{$IFDEF WIN32}
    SLen:= length(S);
    if SLen >= Len then
      Result := S
    else if SLen < 255 then begin
      SetLength(Result, Len);
      Move(S[1], Result[Succ(Word(Len))-SLen], SLen);
      FillChar(Result[1], Len-SLen, Ch);
    end;
{$ELSE}
    if Length(S) >= Len then
      LeftPadCh := S
    else if SLen < 255 then begin
      o[0] := Chr(Len);
      Move(S[1], o[Succ(Word(Len))-SLen], SLen);
      FillChar(o[1], Len-SLen, Ch);
      LeftPadCh := o;
    end;
{$ENDIF}
  end;

  function LeftPad(const S : string; Len : Byte) : string;
    {-Return a string left-padded to length len with blanks}
  begin
    LeftPad:= LeftPadCh(S, ' ', Len);
  end;

  function TrimLead(const S : string) : string;
    {-Return a string with leading white space removed}
{$IFDEF WIN32}
  begin
    Result:= TrimLeft(S);
  end;
{$ELSE}
  var
    I : integer;
  begin
{>>A2.01} Result:= S;

    I := 1;
    while (I <= Length(S)) and (S[I] <= ' ') do
      Inc(I);
    Dec(I);

{>>A 2.01}
    if I > 0 then
      Delete(Result, 1, I);
{<<A 2.01}
  end;
{$ENDIF WIN32}

  function TrimTrail(const S : string) : string;
    {-Return a string with trailing white space removed}
{$IFDEF WIN32}
  begin
    Result:= SysUtils.TrimRight(S);
  end;
{$ELSE}
  var
    SLen : integer;
  begin
{>>A 2.01}
    Result:= S;
{<<A 2.01}
    SLen:= length(S);

    while ( SLen > 0) and (S[SLen] <= ' ') do
      Dec(SLen);

    SetLength(Result, SLen);
  end;
{$ENDIF WIN32}  

{$IFNDEF WIN32}
  function Trim(const S : string) : string;
    {-Return a string with leading and trailing white space removed}
  var
    I : integer;
    SLen : integer;
  begin
{>>A 2.01}
    Result:= S;
{<<A 2.01}
    SLen:= length(S);
    while (SLen > 0) and (S[SLen] <= ' ') do
      Dec(SLen);

    SetLength(Result, SLen);

    I := 1;
    while (I <= SLen) and (S[I] <= ' ') do
      Inc(I);
    Dec(I);

    Delete(Result, 1, I);
  end;
{$ENDIF WIN32}

  function WordCount(const S : string; WordDelims : TChrSet) : longint;
    {-Given a set of word delimiters, return number of words in S}
  var
    I   : longint;
    SLen: longint;
  begin
    Result:= 0;
    I:= 1;
    SLen:= length(S);

    while I <= SLen do begin
      {skip over delimiters}
      while (I <= SLen) and (S[I] in WordDelims) do
        Inc(I);

      {if we're not beyond end of S, we're at the start of a word}
      if I <= SLen then
        Inc(Result);

      {find the end of the current word}
      while (I <= SLen) and not(S[I] in WordDelims) do
        Inc(I);
    end;
  end;

  function ExtractWord(N : longint; const S : string; WordDelims : TChrSet) : string;
    {-Given a set of word delimiters, return the N'th word in S}
  var
    I, Count: longint;
    Len     : longint;
    SLen    : longint;
  begin
    Count:= 0;
    I:= 1;
    Len:= 0;
    ExtractWord:= EmptyStr;
    SLen:= length(S);

    while (I <= SLen) and (Count <> N) do begin
      {skip over delimiters}
      while (I <= SLen) and (S[I] in WordDelims) do
        Inc(I);

      {if we're not beyond end of S, we're at the start of a word}
      if I <= SLen then
        Inc(Count);

      {find the end of the current word}
      while (I <= SLen) and not(S[I] in WordDelims) do begin
        {if this is the N'th word, add the I'th character to Tmp}
        if Count = N then begin
          Inc(Len);
{$IFDEF WIN32}
          SetLength(Result, Len);
          Result[Len] := S[I];
{$ELSE}
          Result[0] := Char(Len);
          Result[Len] := S[I];
{$ENDIF}
        end;

        Inc(I);
      end;
    end;
  end;

{/// File names //////////////////////////////////////////////////////////////}

  function HasExtension(const Name : string; var DotPos : integer) : Boolean;
    {-Return whether and position of extension separator dot in a pathname}
  var
    I : Word;
  begin
    DotPos := 0;
    for I := Length(Name) downto 1 do
      if (Name[I] = '.') and (DotPos = 0) then
        DotPos := I;
{>>C 14-07-1998}
    HasExtension := (DotPos > 0) and (Pos('\', Copy(Name, Succ(DotPos), cbFileExtLen)) = 0);
{<<C 14-07-1998}
  end;

  function DefaultExtension(const Name, Ext : string) : string;
    {-Return a pathname with the specified extension attached}
  var
    DotPos : integer;
  begin
    if HasExtension(Name, DotPos) then
      DefaultExtension:= Name
    else
      DefaultExtension:= Name+'.'+Ext;
  end;

  function ForceExtension(const Name, Ext : string) : string;
    {-Return a pathname with the specified extension attached}
  var
    DotPos : integer;
  begin
    if HasExtension(Name, DotPos) then
      Result := Copy(Name, 1, DotPos)+Ext
    else
      Result := Name+'.'+Ext;
  end;

  function JustExtension(const Name : string) : string;
    {-Return just the extension of a pathname}
  var
    DotPos : integer;
  begin
    if HasExtension(Name, DotPos) then
{>>C 2.03}
      Result:= Copy(Name, Succ(DotPos), cbFileExtLen)
{<<C 2.03}
    else
      SetLength(Result, 0);
  end;

  function JustFilename(const PathName : string) : string;
    {-Return just the filename of a pathname}
  var
    I : Word;
  begin
    I:= Succ(Word(Length(PathName)));

    repeat
      Dec(I);
    until (PathName[I] in DosDelimSet) or (I = 0);

{>>C 15-11-1997}
    Result:= Copy(PathName, Succ(I), length(PathName));
{<<C 15-11-1997}
  end;

  function JustPathname(const PathName : string) : string;
    {-Return just the drive:directory portion of a pathname}
  var
    I : Word;
  begin
    I := Succ(Word(Length(PathName)));
    repeat
      Dec(I);
    until (PathName[I] in DosDelimSet) or (I = 0);

    if I = 0 then
      {Had no drive or directory name}
      SetLength(Result, 0)
    else if I = 1 then
      {Either the root directory of default drive or invalid pathname}
      Result := PathName[1]
    else if (PathName[I] = '\') then begin
      if PathName[Pred(I)] = ':' then
        {Root directory of a drive, leave trailing backslash}
        Result := Copy(PathName, 1, I)
      else
        {Subdirectory, remove the trailing backslash}
        Result := Copy(PathName, 1, Pred(I));
    end else
      {Either the default directory of a drive or invalid pathname}
      Result := Copy(PathName, 1, I);
  end;

  function AddBackSlash(const DirName : string) : string;
    {-Add a default backslash to a directory name}
  begin
    if (DirName<>EmptyStr) and (DirName[Length(DirName)] in DosDelimSet) then
      Result:= DirName
    else
      Result:= DirName+'\';
  end;

  function CleanFileName(const FileName : string) : string;
    {-Return filename with at most 8 chars of name and 3 of extension}
  var
    DotPos : integer;
    NameLen: integer;
{$IFDEF WIN32}
    LeftNameBoundary: integer;
{$ENDIF WIN32}
  begin
{$IFDEF WIN32}
{>>A 2.03}
    if HasExtension(FileName, DotPos) then
    begin
      NameLen:= Pred(DotPos);
      LeftNameBoundary:= NameLen;

      while (LeftNameBoundary>1) do
        if FileName[NameLen] in DosDelimSet then
          Break
        else
          dec(LeftNameBoundary);

      NameLen:= NameLen - LeftNameBoundary + 1;
      Result:= Copy(FileName, LeftNameBoundary, NameLen) +
        Copy(FileName, DotPos, cbFileExtLen + 1);
{<<A 2.03}
    end
      else
{$ELSE}
    if HasExtension(FileName, DotPos) then begin
      {Take the first 8 chars of name and first 3 chars of extension}
      namelen:= Pred(DotPos);
      if namelen > 8 then
        namelen:= 8;
      Result:= Copy(FileName, 1, namelen)+Copy(FileName, DotPos, 4);
    end else
      {Take the first 8 chars of name}
      Result:= Copy(FileName, 1, 8);
{$ENDIF WIN32}
  end;

  function CleanPathName(PathName : string) : string;
    {-Return a pathname cleaned up as DOS will do it}
  var
    I : Word;
  begin
    Result:= EmptyStr;

    {Remove repeated backslashes}
    I := Succ(Word(Length(PathName)));
    repeat
      dec(I);
      if I > 2 then
        if (PathName[I] = '\') and (PathName[I-1] = '\') then
          if (PathName[I-2] <> ':') then
            Delete(PathName, I, 1);
    until I <= 0;

    I := Succ(Word(Length(PathName)));
    repeat
      {Get the next directory or drive portion of pathname}
      repeat
        Dec(I);
      until (PathName[I] in DosDelimSet) or (I = 0);

      {Clean it up and prepend it to output string}
{>>C 2.03}
      Result := CleanFileName(Copy(PathName, Succ(I), cbFileNameLen))+Result;
{<<C 2.03}
      if I > 0 then
      begin
        Result:= PathName[I]+Result;
        Delete(PathName, I, 255);
      end;
    until I <= 0;
  end;

  function FullPathName(const FName : string) : string;
    {-Given FName (known to exist), return a full pathname}
  var
    CurDir : string;
    Cpos : word;
  begin
    Cpos:= Pos(':', FName);

    if Cpos <> 0 then begin
      {Drive letter specified}
      if FName[Succ(Cpos)] = '\' then
        {Complete path already specified}
        Result:= FName
      else
      begin
        {Drive specified, but incomplete path}
        GetDir(Pos(Upcase(FName[1]), 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'), CurDir);
{>>C 2.03}
        Result:= AddBackSlash(CurDir)+Copy(FName, Succ(Cpos), 128);
{<<C 2.03}
      end;
    end else begin
      {No drive specified}
      GetDir(0, CurDir);
      if FName[1] = '\' then
        {Complete path but no drive}
        Result:= Copy(CurDir, 1, 2)+FName
      else
        {No drive, incomplete path}
        Result:= AddBackSlash(CurDir)+FName;
    end;
  end;

{/////////////////////////////////////////////////////////////////////////////}

{>>A 2.01}
  function RemoveCharacter( const S: string; C: char ): string;
    {- Return a string, contains no C chars }
  var
    Idx: word;
  begin
    Result:= NullString;

    for Idx:= 1 to length(S) do
      if S[Idx]<>C then Result:= Result + S[Idx];
  end;
{<<A 2.01}

  function CompactSpaces( S : string ):string;
  {- return a string with all white spaces removed }
  var
    P : word;
  begin
    S:= Trim( S );

    if length(S)>0 then
    begin
      for P:=1 to length(S)-1 do
        if (S[P]=' ') and ( (S[P+1]=' ') or
            ( (P>1) and (S[P-1]=' ') ) ) then
          while( (S[P-1]=' ') or (S[P+1]=' ')) do
            delete( S, P, 1 );
    end; { if }

    Result:= S;
  end; { CompactSpaces }

  function RemoveSpaces( S : string ): string;
    {- Return a string, contains no space chars }
  begin
    Result:= RemoveCharacter(S, ' ');
  end; { RemoveSpaces }
  
  function digiPadCh( D: longint; Ch : Char; Len : Byte) : string;      {*}
    {-Return a string right-padded to length len with ch}
  begin
    digiPadCh:= PadCh( long2str( D ), Ch, Len );
  end;  

  function digiPad( D: longint; Len : Byte) : string;                   {*}
    {-Return a string right-padded to length len with blanks}
  begin
    digiPad:= PadCh( long2str( D ), ' ', Len );
  end;  

  function digiLeftPadCh( D: longint; Ch : Char; Len : Byte) : string;  {*}
    {-Return a string left-padded to length len with ch}
  begin
    digiLeftPadCh:= LeftPadCh( long2str( D ), Ch, Len );
  end;   

  function digiLeftPad( D: longint; Len : Byte) : string;               {*}
    {-Return a string left-padded to length len with blanks}
  begin
    digiLeftPad:= LeftPadCh( long2str( D ), ' ', Len );
  end;   

  function ValidFileName( const FName : string ):boolean;
  begin
    ValidFileName:= ( pos('?',FName)=0 ) and ( pos( '*',FName)=0 );
  end; 

  function JustDriveName( FName : string ):char;
  begin
    FName:= Trim( FName );
    if FName[2]=':' then Result:= FName[1] else
                         Result:= #0;
  end;

  function JustMainFileName( FName : string ):string;
  {- return just a part of filename before point }
  var
    PointPos : word;
  begin
     PointPos:= pos( '.', FName );
     if PointPos=0 then
        Result:= JustFileName( FName )
     else
     begin
        FName:= JustFileName(FName);
        PointPos:= pos('.',FName);
        Result:= Copy( FName, 1, PointPos-1 );
     end; { if else }
  end; { JustMainFileName }

function CompressPath( P : string; Len : byte ): string;
{- compress the patname to form Drive:\...\[dirname]\filename to fit Len }

const
  DelSet : set of char = ['\'];
var
  Parts, PPos, LPos : integer;
  S : string;
  CPS : string;
begin
  if Len<=4 then
  begin
    Result:= P;

    Exit;
  end; { if }

  if length(P)>Len then
  begin
    S:= EmptyStr;

    Parts:= WordCount( P, DelSet );
    if Parts<2 then
    begin
      Result:= Copy( P, length(P)-Len, Len );

      Exit;
    end;

    S:= ExtractWord( 1, P, DelSet )+'\...\';
    Lpos:= length(S)+1;

    S:= S + ExtractWord( Parts, P, DelSet );

    CPS:= EmptyStr;

    PPos:= Parts-1;

    while length(S)+length(CPS)+1<Len do
    begin
      insert( ExtractWord( PPos, P, DelSet )+'\', S, Lpos );
      dec( PPos );
      CPS:= ExtractWord( PPos, P, DelSet );
    end; { while }

    Result:= S;
  end else
    Result:= P;
end; { CompressPath }

{>>A 1.11}
function RemoveBackSlash( P : string ): string;
begin
  while (length(P)>0) and (P[length(P)]='\') do
    delete( P, length(P), 1 ); 

  Result:= P;
end; { RemoveBackSlash }

function TruncLine( S : string; Len : byte ): string;
begin
  Result:= S;

  if length(Result) > Len then
    SetLength(Result, Len);
end; { TruncLine }

function TruncLineRt( S : string; Len : byte ): string;
begin
  if length( S ) > Len then
  begin
    delete( S, 1, length(S)-Len );
    TruncLineRt:= S;
  end
  else
    TruncLineRt:= S;
end; { TruncLine }

{>> 1.1}
function CompareStrings( S1, S2 : string ): TCompareType;
{$IFNDEF WIN32}assembler;{$ENDIF WIN32}
{$IFDEF WIN32}
var
  R: integer;
begin
  R:= CompareStr(S1, S2);
  if R<0 then
    Result:= ctLess
   else
  if R>0 then
    Result:= ctGreater
  else
    Result:= ctEqual;
{$ELSE  WIN32}
asm
  push DS

  xor BX, BX            { assume S1<S2 in temp result }

  les DI, S2            { ES:[DI] points to S2[0] }
  lds SI, S1            { DS:[SI] points to S1[0] }

  mov AH, DS:[SI]       { length of S1 in AH }

  cmp AH, ES:[DI]       { compare lengths of S1 and S2 }

  jb  @@Done            { if len of S1< len of S2 then exit with ctLess }

  inc BX                { now set result to ctEqual }

  xor CX, CX
  mov CL, AH            { number of chars to compare in CX }

  inc DI                { ES:[DI] now points to S2[1] }
  inc SI                { DS:[SI] now points to S1[1] }

  repe cmpsb            { compare until NO MATCH or CX=0 }
  je @@Done             { if match then exit with ctEqual, already in BX }

  mov BL, 02
  ja  @@Done            { Greater ? Return ctGreater }
  xor BX, BX            { Else return ctLess }

@@Done :
  pop DS

  mov AX, BX            { set result from temp store }
{$ENDIF WIN32}
end;

function StringsIsEqual( S1, S2 : string ): boolean;
{$IFNDEF WIN32}assembler;{$ENDIF WIN32}
{$IFDEF WIN32}
begin
  Result:= ( CompareStr(S1, S2) = 0);
{$ELSE  WIN32}
asm
  push DS

  xor BX, BX            { assume failure in temp result }

  les DI, S2            { ES:[DI] points to S2[0] }
  lds SI, S1            { DS:[SI] points to S1[0] }

  mov AH, DS:[SI]       { length of S1 in AH }

  cmp AH, ES:[DI]       { compare lengths of S1 and S2 }

  jne @@Done            { if not equal then exit with False }

  xor CX, CX
  mov CL, AH            { number of chars to compare in CX }

  inc DI                { ES:[DI] now points to S2[1] }
  inc SI                { DS:[SI] now points to S1[1] }

  repe cmpsb            { compare until NO MATCH or CX=0 }
  ja @@Done             { if no match then exit with False }

  mov BX, 01            { match, returns True }

@@Done :
  pop DS

  mov AX, BX            { set result from temp store }
{$ENDIF WIN32}
end;

{/////////////////////////////////////////////////////////////////////////////}

{>> 1.12}
function FormatFIO( SrName, FName, SName : string ): string;
begin
  SrName:= trim( SrName );
  FName:=  trim( FName );
  SName:=  trim( SName );

  Result:= SrName;
  if FName<>NullString then Result:= Result + ' ' + FName[1] + '.';
  if SName<>NullString then Result:= Result + SName[1] + '.';
end;

function FormAsName(S:string): string;                        {*}
  {-Make first letter in string as uppercase }
begin
  S:= ANSIlowerCase( S );
  S[1]:= ANSIupperCase( S[1] )[1];
  Result:= S;
end; { FormAsName }

function FormAmericanString(S:string) : string;               {*}
  {-Make first letters in line as uppercase }
var P : byte;
begin
  S[1]:= ANSIupperCase( S[1] )[1];

  for P:=1 to length(S)-1 do
    if (S[P]=' ') and (S[P+1]<>' ') then S[P+1]:= ANSIupperCase(S[P+1])[1];

  Result:= S;
end; { FormAmericanString }

function CompStruct(fpBlock1, fpBlock2: Pointer; Size: Cardinal): Boolean; assembler;
asm
{$IFDEF WIN32}
        PUSH    ESI
        PUSH    EDI
        MOV     ESI,fpBlock1
        MOV     EDI,fpBlock2
        MOV     EDX,ECX
        XOR     EAX,EAX
        AND     EDX,3
        SHR     ECX,2
        REPE    CMPSD
        JNE     @@2
        MOV     ECX,EDX
        REPE    CMPSB
        JNE     @@2
@@1:    INC     EAX
@@2:    POP     EDI
        POP     ESI
{$ELSE  WIN32}
        PUSH    DS
        LDS     SI,fpBlock1
        LES     DI,fpBlock2
        MOV     CX,Size
        XOR     AX,AX
        CLD
        REPE    CMPSB
        JNE     @@1
        INC     AX
@@1:    POP     DS
{$ENDIF}
end; { CompStruct }

{$IFNDEF WIN32}
procedure SetLength(var S: string; Len: byte);
begin
  S[0]:= char(Len);
end;
{$ENDIF}

{>>A 2.01}
function  UpCaseFirstLtr(const S: string): string;
begin
  Result:= S;
  if length(Result)<=0 then Exit;

  Result[1]:= ANSIupperCase(Result[1])[1];
end; { UpCaseFirstLtr }

function  nxStrToInt(const S: string): longint;
begin
  if trim(S) = EmptyStr then Result:= 0
    else Result:= StrToInt(S);
end; { nxStrToInt }

function  StrIsNull(const S: string): boolean;
begin
  Result:= (length(trim(S))<=0);
end; { StrIsNull }
{<<A 2.01}

{>>A 2.02}
function  CharCount(C: char; const S: string): integer;
var
  Idx: integer;
begin
  Result:= 0;

  for Idx:= 1 to length(S) do
    inc(Result, byte( S[Idx]=C ));
end; { CharCount }

procedure ReplaceChar(SrcChar, DstChar: char; var S: string);
{- replaces all occurencies of SrcChar in string S by DstChar }
var
  I: integer;
begin
  for I:= 1 to length(S) do
    if S[I] = SrcChar then
      S[I]:= DstChar;
end; { ReplaceChar }

function  GetQuotedString(const S: string): string;
{- returns string inside double quotes (long path names, etc.). If no one
  double qoutes apears in S then entire S will be returned as a result.
  If only one double quote appears in S all remaining characters will be
  returned as a result. }
var
  QPos: {$IFDEF DELPHI3}longint{$ELSE}word{$ENDIF};
begin
  QPos:= pos(DoubleQuote, S);

  if QPos<=0 then
  begin
    Result:= S;

    Exit;
  end; { if }

  Result:= S;
  delete(Result, 1, QPos+1);

  QPos:= pos(DoubleQuote, Result);

  if QPos>0 then
    Result:= copy(Result, 1, QPos-1); 
end; { GetQuotedString }

procedure ClearTChar(var TC: TChar);
begin
  FillChar(TC, SizeOf(TChar), 0);
end; { ClearTChar }

function  HexToInt(const H: string; Digits: integer; var I: longint): boolean;
{- Converts a hexadecimal string to long integer. Digits determines how
  much digits in a string must be used during conversion. Returns False if
  empty string passed or non-hex characters encountered. If error occurs,
  I variable does not changes it's value. In case of success returns True
  and decimal value in I variable. }
var
  Idx: integer;
  L: longint;
  H2B: byte;
  Pwr: byte;

  function HexPower(Signer: integer): longint;
  var
    HxIdx: integer;
  begin
    Result:= 1;
    if Signer=0 then
      Exit;

    Result:= 1;
    for HxIdx:= 1 to Signer do
      Result:= Result * 16;
  end; { HexPower }

begin
  Result:= False;
  L:= 0;

  for Idx:= 1 to length(H) do
    if pos(UpCase(H[Idx]), HEXdigits) = 0 then Exit;

  if Digits>8 then
    Exit;

  Pwr:= 0;
  for Idx:= length(H) downto 1 do
  begin
    for H2B:= 0 to $F do
      if HEXdigits[H2B] = UpCase(H[Idx]) then
      begin
        L:= L + HexPower(Pwr) * H2B;

        Break;
      end; { if }

    inc(Pwr);  
    dec(Digits);

    if Digits<=0 then
      Break;
  end; { for }

  I:= L;

  Result:= True;  
end; { HexToInt }

{<<A 2.02}
{/////////////////////////////////////////////////////////////////////////////}

end.

{<< EOF>>}