unit PPROGRSS;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Gauges, StdCtrls, Buttons;

type
  TNXprintProgressDlg = class(TForm)
    Gauge: TGauge;
    CancelBtn: TBitBtn;
    MessageLbl: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CancelBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    CancelPressed: boolean;
  end;

function CreateNXProgressDlg( Caption : string; Msg : string;
                              MinValue, MaxValue, Progress : longint ): boolean;

function DestroyNXProgressDlg: boolean;

function UpdateNXProgressDlg( Msg : string; Progress : longint;
                              CancelPressed: boolean ): boolean;

implementation

{$R *.DFM}

uses
  NxStrUtl;

var
  NXPDlg: TNXprintProgressDlg;


function CreateNXProgressDlg;
begin
  Result:= False;

  if assigned(NXPdlg) then
  begin
    try
      NXPdlg.Free;
    except
      Exit;
    end; { try }
  end; { if }

  try
    NXPdlg:= TNXprintProgressDlg.Create( Application );
  except
    MessageDlg('     !',
               mtError,
               [mbOK], 0 );

    Exit;
  end; { try }

  Application.ProcessMessages;

  if GetCapture <> 0 then SendMessage(GetCapture, WM_CANCELMODE, 0, 0);

  try
    NXPdlg.MessageLbl.Caption:= Msg;
    NXPdlg.Gauge.MinValue:= MinValue;
    NXPdlg.Gauge.MaxValue:= MaxValue;
    NXPdlg.Gauge.Progress:= Progress;

    NXPdlg.Cursor:= crDefault;
    NXPdlg.CancelBtn.Cursor:= crDefault;
    Application.ProcessMessages;

    NXPdlg.Show;

  except
    MessageDlg('     !',
               mtError,
               [mbOK], 0 );

    try
      NXPdlg.Free;
    finally
      NXPdlg:= NIL;
    end; { try }

    Exit;
  end; { try }

  Result:= True;
end;

function DestroyNXProgressDlg: boolean;
begin
  Result:= False;

  if assigned( NXPdlg ) then
    try
      ReleaseCapture;
      NXPdlg.Free;
    except
      Exit;
    end; { try }

  NXPdlg:= NIL;

  Application.ProcessMessages;

  Result:= True;
end;

function UpdateNXProgressDlg;
begin
  Result:= False;

  if not Assigned( NXPdlg ) then Exit;

  if trim(Msg)<>NullString then
    NXPdlg.MessageLbl.Caption:= Msg;

  NXPdlg.Gauge.Progress:= Progress;

  NXPdlg.Update;

  Application.ProcessMessages;

  CancelPressed:= NXPdlg.CancelPressed;

  Result:= True;
end;

{/////////////////////////////////////////////////////////////////////////////}

procedure TNXprintProgressDlg.FormCreate(Sender: TObject);
begin
  CancelPressed:= False;
end;

procedure TNXprintProgressDlg.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action:= caFree;
end;

procedure TNXprintProgressDlg.CancelBtnClick(Sender: TObject);
begin
  CancelPressed:= True;
end;

initialization
  NXPdlg:= NIL;

end.
